#include "RPlot.h"

#include <qstring.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qtextedit.h>

#include <iostream>

using namespace std;

RPlot::RPlot ( Rinterface *pipe, QWidget* parent, const char* name, 
               bool modal, WFlags fl )
  : RPlotFormBase( parent, name, modal, fl ) {

  // set the connection to R
  this->pipe = pipe;

  // set the defaults
  connectionEstablished = false;
  dataLoaded = false;
  sourceRFile = false;
  isConfigured = false;

  cout << "in constructor" << endl;

  // connect the signals
  connect(sourceFileButton, SIGNAL(clicked()), 
          this, SLOT(setSourceFile()));
  connect(dataFileButton, SIGNAL(clicked()), 
          this, SLOT(setDataFile()));
  connect(outputFileButton, SIGNAL(clicked()),
          this, SLOT(setOutputFile()));
  connect(sourceFileEdit, SIGNAL(textChanged( const QString & )),
          this, SLOT(setSourcing( const QString &)));
  connect(dataFileEdit, SIGNAL(textChanged( const QString & )),
          this, SLOT(setDataLoading( const QString &)));
  connect(configureRButton, SIGNAL(clicked()),
          this, SLOT(configureForPlotting() ));
}

// the do nothing constructor
RPlot::~RPlot() { /* NOOP */ }

// if text has changed in the source file edit line make sure that the command is resourced
void RPlot::setSourcing(const QString &ignoreme) {
  sourceRFile = true;
  isConfigured = false;
}

// if the text inside the data file box has changed then the data needs to be reloaded.
void RPlot::setDataLoading(const QString &ignoreme) {
  dataLoaded = false;
  isConfigured = false;
}

// opens a file dialog to get a filename
void RPlot::setSourceFile() {
  QString s = QFileDialog::getOpenFileName();
  sourceFileEdit->setText(s);
}

// opens a dialog to get a filename
 void RPlot::setOutputFile() {
   QString s = QFileDialog::getOpenFileName();
   outputFilenameEdit->setText(s);
 }

// opens a file dialog to get a filename
void RPlot::setDataFile() {
  QString s = QFileDialog::getOpenFileName();
  dataFileEdit->setText(s);
}

// a slot to receive the signal from the pointPicker. Plots a vertex - if that's what the user wants
//void RPlot::plotFromPickedPoint( float x, float y, float z, int i) {
//  if (plotCheck->isChecked() == true && isConfigured == true) {
//    plotVertex(i);
//  }
//}

// establishes the connections with the R-server pipe
bool RPlot::establishConnection() {
  // only do something if the connection has not already been established
  if (connectionEstablished == false) {
    
    Rinterface::connectionStatus s = pipe->establishConnection();
    cout << "after connection establishment" << endl;
    if (s == Rinterface::CONNECTIONERROR) {
      cout << "in exception" << endl;
      QMessageBox::critical( 0, 
                             tr("Fatal Error"), 
                             tr("Could not start the connection to R"),
                             tr("Quit") );
      return(false);
    }
    // mixed model plots and lattice graphics
    pipe->sendCommand(tr("library(nlme)")); 
    cout << "connection established" << endl;

    connectionEstablished = true;
    
    return(true);
  
  }
}


// sources files and, if need be, loads data into R
void RPlot::configureForPlotting() {

  cout << "in configure for plotting" << endl;

  // make sure that the pipe is happy
  if (connectionEstablished == false) {
    establishConnection();
  }

  // load the data if need be
  if (dataLoaded == false && loadDataCheck->isChecked() == true) {
    pipe->loadDataFile( dataFileEdit->text().ascii() );
    dataLoaded = true;
  }

  // source the R file if need be
  if (sourceRFile == true) {
    QString sourceCommand;
    sourceCommand.sprintf( "source(\"%s\")", sourceFileEdit->text().ascii() );
    pipe->sendCommand(sourceCommand, false);
    sourceRFile = false;
  }
  
  isConfigured = true;

}
    
// plot an actual point
void RPlot::plotVertex( int vertexNumber ) {
  if (plotCheck->isChecked() == true && isConfigured == true) {
    // assume that the vertex number is from 0 based counting ... R uses 1 based.
    vertexNumber++;
    QString vNumber;
    vNumber.sprintf("v <- %i", vertexNumber);
    pipe->sendCommand(vNumber);
    
    // set the output file and related bits if format other than window was chosen
    if (pdfButton->isChecked()) {
      // Todo: make sure that a filename has actually been specified
      QString command;
      command.sprintf("trellis.device(\"pdf\", height = %s, width = %s, file =\"%s\")",
                      heightEdit->text().ascii(), widthEdit->text().ascii(),
                      outputFilenameEdit->text().ascii());
      pipe->sendCommand(command);
      pipe->sendCommand("f <- trellis.par.get(\"fontsize\")");
      command.sprintf("f$default <- %s", fontsizeEdit->text().ascii() );
      pipe->sendCommand(command);
      pipe->sendCommand("trellis.par.set(\"fontsize\", f)");

    }
      

    // get the plotting command
    QString plotCommand = plotCommandEdit->text();
    pipe->sendCommand(plotCommand);

    if (pdfButton->isChecked()) {
      pipe->sendCommand("dev.off()");
    }

  }
}


